#include "esp_system.h"
#include "esp_log.h"
#include "nvs_flash.h"
#include "driver/gpio.h"

#include <stdio.h>
#include <math.h>

volatile float f1 = 1.11111111111111111111;
volatile float f2 = 4.44444444444444444444;

volatile float fSum;
volatile float fSqr;
volatile float fMul;
volatile float fDiv;
volatile float fPow;
volatile float fSin;
volatile float fLst;

volatile double d1 = 1.11111111111111111111;
volatile double d2 = 4.44444444444444444444;

volatile double dSum;
volatile double dSqr;
volatile double dMul;
volatile double dDiv;
volatile double dPow;
volatile double dSin;
volatile double dLst;


// Quick hack to print floating-point values
static char *f2s(char *buf, float f);
static char *d2s(char *buf, double f);
char gBuf[256], gBuf2[256];


int app_main(void)
{
    nvs_flash_init();

    gpio_set_direction(GPIO_NUM_18, GPIO_MODE_OUTPUT);
    gpio_set_level(GPIO_NUM_18, 0);

    ESP_LOGI("main", "starting");

    while (1) {
        // 236ns
        fSum = f1 + f2;
        // 4.6us
        fSqr = sqrtf(f1);
        // 226ns
        fMul = f1 * f2;
        // 424ns
        fDiv = f1 / f2;
        // 15.1us
        fPow = powf(f1, f2);
        // 1.1us
        fSin = sinf(f1);

        // 490ns
        dSum = d1 + d2;
        // 4.2us
        dSqr = sqrt(d1);
        // 680ns
        dMul = d1 * d2;
        // 2.2us
        dDiv = d1 / d2;
        // 71us
        gpio_set_level(GPIO_NUM_18, 1); // Set GPIO 18 HIGH
        dPow = pow(d1, d2);
        gpio_set_level(GPIO_NUM_18, 0); // Set GPIO 19 LOW
        // 8.8us
        dSin = sin(d1);
        
        printf("--------- float  ---------------------------------------\r\n");
        printf("f1   = %s, f2   = %s\r\n", f2s(gBuf, f1), f2s(gBuf2, f2));
        printf("fSum = %s, fSqr = %s\r\n", f2s(gBuf, fSum), f2s(gBuf2, fSqr));
        printf("fMul = %s, fDiv = %s\r\n", f2s(gBuf, fMul), f2s(gBuf2, fDiv));
        printf("fPow = %s, fSin = %s\r\n", f2s(gBuf, fPow), f2s(gBuf2, fSin));
        
        printf("--------- double ---------------------------------------\r\n");
        printf("d1   = %s, d2   = %s\r\n", d2s(gBuf, d1), d2s(gBuf2, d2));
        printf("dSum = %s, dSqr = %s\r\n", d2s(gBuf, dSum), d2s(gBuf2, dSqr));
        printf("dMul = %s, dDiv = %s\r\n", d2s(gBuf, dMul), d2s(gBuf2, dDiv));
        printf("dPow = %s, dSin = %s\r\n", d2s(gBuf, dPow), d2s(gBuf2, dSin));
        printf("dLst = %20.18f\r\n", dLst);
    }
}

char *f2s(char *buf, float f)
{
    if (f >= 1.0) {
        int h = (int)f;
        f -= h;
        buf[0] = '0' + h;
    } else {
        buf[0] = '0';
    }
    buf[1] = '.';
    int ix = 2;
    
    while (f > 0 && ix < 20) {
        f = f * 10;
        int x = (int)f;
        f = f - x;
        buf[ix++] = '0' + x;
    }
    
    buf[ix] = 0x00;
    return buf;
}

char *d2s(char *buf, double f)
{
    if (f >= 1.0) {
        int h = (int)f;
        f -= h;
        buf[0] = '0' + h;
    } else {
        buf[0] = '0';
    }
    buf[1] = '.';
    int ix = 2;
    
    while (f > 0 && ix < 20) {
        f = f * 10;
        int x = (int)f;
        f = f - x;
        buf[ix++] = '0' + x;
    }
    
    buf[ix] = 0x00;
    return buf;
}
