#include "stm32f7xx.h"
#include <math.h>

volatile double d1 = 1.11111111111111111111;
volatile double d2 = 4.44444444444444444444;
volatile double dSum;
volatile double dSqr;
volatile double dMul;
volatile double dDiv;
volatile double dPow;
volatile double dSin;
volatile double dLst;

static int initCpuClock();

// LED2 (blue) = PB7
int main()
{
	initCpuClock();

	RCC->AHB1ENR |= 0x0000000E; // Enable clock for GPIOB, GPIOC, GPIOD
	GPIOB->MODER |= 0x10004000; // set PB7 and PB14 as output

	// SysTick: program reload value, clear current value, program control + status reg
	*(unsigned long *)0xE000E014 = *(unsigned long *)0xE000E01C; // reload
	*(unsigned long *)0xE000E018 = 0; // current
	*(unsigned long *)0xE000E010 = 0x00000007; // control+status


	// Enable Interrupts
	//__asm volatile ("cpsie i" : : : "memory");

	while (1) {
		// Measurement of FPU operations

		// 50ns
		//GPIOB->BSRR = 0x00000080; // set led 2 ON
		dSum = d1 + d2;
		//GPIOB->BSRR = 0x00800000; // set led 2 OFF	

		// 390ns
		//GPIOB->BSRR = 0x00000080; // set led 2 ON
		dSqr = sqrt(d1);
		//GPIOB->BSRR = 0x00800000; // set led 2 OFF	

		// 50ns
		//GPIOB->BSRR = 0x00000080; // set led 2 ON
    		dMul = d1 * d2;
		//GPIOB->BSRR = 0x00800000; // set led 2 OFF	

		// 160ns
		//GPIOB->BSRR = 0x00000080; // set led 2 ON
    		dDiv = d1 / d2;
		//GPIOB->BSRR = 0x00800000; // set led 2 OFF	

		// 7400ns
		GPIOB->BSRR = 0x00000080; // set led 2 ON
    		dPow = pow(d1, d2);
		GPIOB->BSRR = 0x00800000; // set led 2 OFF	

		// 2600ns
		//GPIOB->BSRR = 0x00000080; // set led 2 ON
   		dSin = sin(d1);
		//GPIOB->BSRR = 0x00800000; // set led 2 OFF	
	}
}

static int initCpuClock()
{
	// Enable clock for Power Interface
	RCC->APB1ENR |= 0x10000000U; // RCC_APB1ENR_PWREN

	PWR->CR1 |= 0x0000C000U; // 11 = voltage scale 1


	// Prepare PLL
	// PLL_R = 7, PLL_Q = 9 (1001), src=HSI (0), PLL_P = 2 (00),
	// PLL_N = 432 (max!), PLL_M = 16, 
	RCC->PLLCFGR = 0x79006C10U;
	// Bit 25: PLL ready, Bit 24: PLL enabled
	RCC->CR |= 0x01000000U;

	// Set ODEN in PWR_CR1 and wait for ODRDY
	PWR->CR1 |= 0x00010000U;
	while (! (PWR->CSR1 & 0x00010000U) ) {
		// NOP -> todo: timeout (how long?)
	}

	// Set ODSWEN in PWR_CR1
	PWR->CR1 |= 0x00020000U;

	// Wait for ODSWRDY
	while (! (PWR->CSR1 & 0x00020000U) ) {
		// NOP -> todo: timeout (how long?)
	}

	// Select flash latency (3.3.2, table 7) and AHB/APB prescalers
	// FLASH_ACR FLASH_ACR_LATENCY_7WS
	// Write then read wait states
	FLASH->ACR |= 7; // 7 wait states
	while ( (FLASH->ACR & 0xf) != 7) {
		// NOP -> todo: timeout (how long?)
	}
	// AHB /1, APB1 /4 (54 MHz), APB2 /2 (108 MHz), still using HSI
	RCC->CFGR = 0x00009400U;

	// ART accelerator, prefetch
	// ASR: Tested with these flags enabled, no difference.
	// *(unsigned long *)FLASH_ACR |= 0x00000300;

	// Wait for PLL Lock
	while (! (RCC->CR & 0x02000000U) ) {
		// NOP -> todo: timeout (how long?)
	}

	// switch system clock to PLL, wait until switch is complete
	RCC->CFGR |= 0x00000002U;
	while (8 != (RCC->CFGR & 0x0000000CU) ) {
		// NOP -> todo: timeout
	}

	return 1;
}

volatile int x = 0;

void SysTick_Handler()
{
	if (x == 1000) {
		x = 0;
	}
	x++;
}




